xquery version "3.1";

(:~
 : This module provides the API endpoint for kmg-print.
 :
 : @version 1.0
 : @status stable
 : @author Max-Ferdinand Zeterberg
 :)
module namespace rest-print="https://mollenhauer-edition.de/ns/rest-print";

import module namespace config="https://sade.textgrid.de/ns/config" at "../../config.xqm";
import module namespace selection = "https://mollenhauer-edition.de/ns/conversions/selection" at "selection.xqm";
import module namespace simplify="https://mollenhauer-edition.de/ns/conversions/simplify" at "simplify.xqm";
import module namespace rest="http://exquery.org/ns/restxq";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace output ="http://www.w3.org/2010/xslt-xquery-serialization";

(: ~
 : This functions allows to request data for print publication.
 :
 : @param $kmg KMG-Number
 : @return A XML object (simplified TEI-XML)
 :)
declare
    %rest:path("api/kmg4print")
    %rest:GET
    %rest:form-param("kmg", "{$kmg}")
    %output:method("xml")
function rest-print:kmg4print($kmg as xs:string*) as item()*
{
    let $uri := selection:TGURI-from-KMG-number($kmg)
    let $doc := doc($config:sade-data || $uri || '.xml')
    let $versionID := (
        if ( $doc//tei:textClass[@corresp="#KMG-Text"]//tei:term/@key = 'documenttype:da6784cb-a922-4ddf-aca1-dc96b24f07d4' ) then
            data($doc//tei:sourceDesc/tei:listWit/tei:witness[last()]/@xml:id)
        else
            data($doc//tei:sourceDesc/tei:listWit/tei:witness[1]/@xml:id)
    )
    return
        simplify:all($doc, $versionID)
};