xquery version "3.1";

module namespace kmg-edition-nav="https://mollenhauer-edition.de/ns/edition-nav";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace xhtml="http://www.w3.org/1999/xhtml";

import module namespace config="https://sade.textgrid.de/ns/config" at "../config.xqm";
import module namespace transformKMG="http://mollenhauer-edition.de/ns/transform" at "transform.xqm";

declare function kmg-edition-nav:get-editions($node as node(), $model as map(*))
{
    for $doc in collection($config:sade-data)
    order by
        substring($doc//tei:title[@type="main"]/text(), 1, 1)
    return
        kmg-edition-nav:entry($doc)
};

declare function kmg-edition-nav:entry($doc as node()*)
as element(toggle-section)*
{
    let $year := tokenize($doc//tei:sourceDesc//tei:desc[contains(@type, "Leittext")]/text(), "\s")[last()]
    let $year := if ( contains($year, "-") ) then substring-before($year, "-") else $year
    let $title := transformKMG:transform($doc//tei:title[@type="main"], "navrequest", "none")
    let $versions :=  $doc//tei:sourceDesc/tei:listWit/tei:witness
    let $uri := replace(util:document-name($doc), '\..*', '')
    let $target := "kmg.html?file=" || $uri || "&amp;edition="
    return
        element toggle-section
        {
            attribute data-keywords
            {
                string-join($doc//tei:profileDesc//tei:keywords[not(contains(@scheme, "documenttype"))]/tei:term, " ")
            },
            attribute data-genre
            {
                string-join($doc//tei:profileDesc//tei:keywords/tei:term/@key, " ")
            },
            attribute data-year
            {
                $year
            },
            element h2
            {
                attribute id { replace(string-join($title), " ", "-") },
                $title || " (" || $year || ")"
            },
            element ul
            {
                if (count($versions) = 1) then
                    element li
                    {
                        attribute class { "edition-nav-li" },
                        element a
                        {
                            attribute href { $target || "a" },
                            element span {
                                attribute class { "icon-listarrow" }
                            },
                            $versions[1]/tei:desc
                        }
                    }
                else
                    (element li
                    {
                        attribute class { "edition-nav-li" },
                        element a
                        {
                            attribute href { $target || "digital" },
                            element span {
                                attribute class { "icon-listarrow" }
                            },
                            "Textkritische interaktive Ansicht mit a als Leittext"
                        }
                    },
                    for $version in $versions
                    return
                        element li
                        {
                            attribute class { "edition-nav-li" },
                            element a
                            {
                                attribute href { $target || $version/@xml:id },
                                element span {
                                    attribute class { "icon-listarrow" }
                                },
                                "Fassung " || $version/@xml:id || ": " || $version/tei:desc
                            }
                        }
                    ),
                element li
                {
                    attribute class { "edition-nav-li" },
                    element a
                    {
                        attribute href { $target || "commentary" },
                        element span {
                            attribute class { "icon-listarrow" }
                        },
                        "Werkkommentar"
                    }
                }
            }
        }
};

declare function kmg-edition-nav:get-keywords($node as node(), $model as map(*))
{
    let $keywords := kmg-edition-nav:find-unique-keywords()
    for $keyword in $keywords[index-of($keywords, .)[1]]
    return
        element option
        {
            attribute value {$keyword/text()},
            $keyword/text()
        }
};

declare function kmg-edition-nav:find-unique-keywords()
{
    for $doc in collection($config:sade-data)
    let $keywords := $doc//tei:profileDesc//tei:keywords[not(contains(@scheme, "documenttype"))]/tei:term
    return
        $keywords
};

declare function kmg-edition-nav:get-genres($node as node(), $model as map(*))
{
    let $genres := kmg-edition-nav:find-unique-genres()
    for $genre in $genres[index-of($genres, .)[1]]
    return
        element option
        {
            attribute value {$genre/@key},
            $genre/text()
        }
};

declare function kmg-edition-nav:find-unique-genres()
{
    for $doc in collection($config:sade-data)
    let $genres := $doc//tei:profileDesc//tei:keywords[contains(@scheme, "documenttype")]/tei:term
    return
        $genres
};
