xquery version "3.1";

 (:~
 : This module generates register as TEI-XML for person entities from the editions-TEI
 : and for artworks from lido-XML
 : @author Florian Barth
 : @version 1.0
 : @since 1.0
 :)


module namespace kmg-register-transformation="https://mollenhauer-edition.de/ns/register-transformation";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace pnd="http://textgrid.info/namespaces/vocabularies/pnd";
declare namespace lido="http://www.lido-schema.org";


declare function kmg-register-transformation:persons-all-references($sade-data-path as xs:string*){

    for $doc in collection($sade-data-path)
    let $persons := $doc//tei:text//tei:name[@type='person']
    return
        for $person in $persons
        return
            (: if $person/@ref is a reference to xml:id replace the hashtag
            and search for xml:id :)
            if (matches($person/@ref, '^#')) then
                    $doc//.[@xml:id=replace($person/@ref, '#', '')]
            else
                $person
};



declare function kmg-register-transformation:persons-unique-references($sade-data-path as xs:string*){

    for $person_ref in distinct-values(kmg-register-transformation:persons-all-references($sade-data-path)/@ref)
    let $stripped_id := tokenize($person_ref, ":")[2]
    return
        try{
            <person xmlns="http://www.tei-c.org/ns/1.0" ref="{$person_ref}">
                <persName xml:lang="de">{doc("https://ref.de.dariah.eu/pndsearch/pndquery.xql?id=" || $stripped_id)//pnd:si[1]/text()}</persName>
            </person>
        } catch * {
            <person xmlns="http://www.tei-c.org/ns/1.0" ref="{$person_ref}">
                <persName xml:lang="de">_Wrong GND-id format!</persName>
            </person>
        }
};


declare function kmg-register-transformation:artworks-all-titles($sade-data-path as xs:string*){
    for $doc in collection($sade-data-path)
    let $title := $doc//lido:titleSet/lido:appellationValue[@xml:lang="de"]/text()
    let $tg-id := $doc//lido:lidoRecID/text() => substring-before(".")
    return
        <item xmlns="http://www.tei-c.org/ns/1.0" ref="{$tg-id}" type="art">
            <name xml:lang="de">{$title}</name>
        </item>

};

declare function kmg-register-transformation:return-register($sade-data-path as xs:string*, $register-type as xs:string*){
    let $body :=
        if ($register-type = "persons")
        then
            <body xmlns="http://www.tei-c.org/ns/1.0">
                <listPerson type="person">
                    {kmg-register-transformation:persons-unique-references($sade-data-path)}
                </listPerson>
            </body>
        else if ($register-type = "artworks")
        then
            <body xmlns="http://www.tei-c.org/ns/1.0">
                <list type="art">
                    {kmg-register-transformation:artworks-all-titles($sade-data-path)}
                </list>
            </body>
        else ()

    let $register :=
        <TEI xmlns="http://www.tei-c.org/ns/1.0">
            <teiHeader>
                <fileDesc>
                    <titleStmt>
                        <title>Register for {$register-type}</title>
                        <author>
                            <name ref="gnd:118583352" type="person">Klaus Mollenhauer</name>
                        </author>
                        <funder>Deutsche Forschungsgemeinschaft (DFG)</funder>
                        <respStmt>
                            <persName>
                                <forename/>
                                <surname/>
                            </persName>
                            <resp/>
                        </respStmt>
                    </titleStmt>
                    <editionStmt>
                        <edition>
                            Historisch-kritische Edition
                        </edition>
                    </editionStmt>
                    <extent>File size</extent>
                    <publicationStmt>
                        <publisher>
                            <name type="org">Niedersächsische Staats- und
                        Universitätsbibliothek Göttingen</name>
                        </publisher>
                        <pubPlace>
                            <name ref="geo:2918632" type="place" xml:id="GÖ">Göttingen</name>
                        </pubPlace>
                        <date type="publication"/>
                        <availability>
                            <licence target="https://creativecommons.org/licenses/by-sa/4.0/">Distributed under the Creative Commons                                 Attribution-ShareAlike 4.0 Generic (International) License.</licence>
                        </availability>
                    </publicationStmt>
                </fileDesc>
                <encodingDesc>
                    <projectDesc>
                        <p>Klaus Mollenhauer Gesamtausgabe</p>
                    </projectDesc>
                    <editorialDecl>
                        <correction>
                            <p>whether corrections are encoded or not</p>
                        </correction>
                        <hyphenation>
                            <p>whether we respect the caesura at the end of lines</p>
                        </hyphenation>
                        <normalization>
                            <p>Information about abbreviations</p>
                        </normalization>
                    </editorialDecl>
                </encodingDesc>
                <profileDesc>
                    <langUsage>
                        <language ident="deu">Deutsch</language>
                    </langUsage>
                </profileDesc>
            </teiHeader>
            <text>
                <front>
                    <p>This register is about all named {$register-type}.</p>
                </front>

                {$body}
            </text>
        </TEI>

        return
            document {
                processing-instruction xml-model {
                    'href="http://www.tei-c.org/release/xml/tei/custom/schema/relaxng/tei_bare.rng" type="application/xml" schematypens="http://purl.oclc.org/dsdl/schematron"'
                },
                $register
            }
};
