xquery version "3.1";

module namespace kmg-register-view="https://mollenhauer-edition.de/ns/register-view";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace xhtml="http://www.w3.org/1999/xhtml";

import module namespace config="https://sade.textgrid.de/ns/config" at "../config.xqm";


(:~
 : This function returns a list of register entries of entities (persons, artworks)
 : filtered by a selected letter (from button in register.html)
 :
 : @param $entityButtonValue: value of the input element from toolbar for entities (persons, artworks)
 : @param $letterButtonValue: value of the input element from toolbar for letters (persons, artworks)
 : @param $letterButtonText: text of the input element from toolbar for letters (persons, artworks)
 : @return $doc A TEI-XML document with the full Series Statement
 :)
declare function kmg-register-view:display-register($entityButtonValue as xs:string*, $letterButtonValue as xs:string*, $letterButtonText as xs:string*) as element(xhtml:div)*{

    let $entities :=
        if ($entityButtonValue = "persons")
        then
            doc($config:register || $entityButtonValue || ".xml")//tei:person
        else if ($entityButtonValue = "artworks")
        then
            doc($config:register || $entityButtonValue || ".xml")//tei:item
        else ()

    let $classAttribute :=
        if ($entityButtonValue = "persons")
        then
            "js-editorial-info--person"
        else if ($entityButtonValue = "artworks")
        then
            "js-editorial-info--art"
        else ()

    let $letterQuery := "^[" || $letterButtonValue || "]"



    return
        <xhtml:ul>
        <h2>{string($letterButtonText)}</h2>
        {
        for $entity in $entities
        where matches(string($entity), $letterQuery)
        return
            element xhtml:li {
                element xhtml:span {
                    attribute class {$classAttribute},
                    if ($entityButtonValue = "persons")
                    then
                        (attribute data-target {substring-after($entity/@ref, ":")},
                        $entity/tei:persName/text())
                    else if ($entityButtonValue = "artworks")
                    then
                        (attribute data-target {substring-after($entity/@ref, ":") => replace("\..*", "")},
                        $entity/tei:name/text())
                    else()
                }
            }

        }
        </xhtml:ul>

};
