xquery version "3.1";

module namespace kmg-register="https://mollenhauer-edition.de/ns/register";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace xhtml="http://www.w3.org/1999/xhtml";

import module namespace config="https://sade.textgrid.de/ns/config" at "../config.xqm";

declare function kmg-register:get-registers($node as node(), $model as map(*))
{
    for $doc in collection($config:register)
    return
        kmg-register:get-register($doc)
};

declare function kmg-register:get-register($doc as node()*)
{
    element noscript
    {
        element h2
        {
            switch ($doc//tei:body/child::*[1]/@type)
            case "person"
            return
                "Personenregister"
            case "art"
            return
                "Register der Kunstwerke"
            case "musicalworks"
            return
                "Register der Musikstücke"
            default
            return
                "Register"
        }
    },
    element ul
    {
        attribute class { "index-entries" },
        attribute data-indextype { $doc//tei:body/child::*[1]/@type },
        kmg-register:get-entries($doc)
    }
};

declare function kmg-register:get-entries($doc as node()*)
{
    for $entry in kmg-register:sort-entries($doc)
    let $type := $doc//tei:body/child::*[1]/@type
    return
        element li
        {
            attribute class { "js-editorial-info--" || $type },
            attribute data-letter { substring($entry/child::*/text(), 1, 1) => lower-case()},
            attribute data-target { $entry/@ref => substring-after(":") },
            $entry//text()
        }
};

declare function kmg-register:sort-entries($doc as node()*)
{
    for $entry in $doc//tei:body//tei:person|$doc//tei:body//tei:item
    where substring($entry/child::*/text(), 1, 1) ne "_"
    order by $entry/child::*/text()
    return
        $entry
};