(:~
 : This module serves for getting all the XML serialization and its respective
 : preparation done.
 :
 : For displaying the XML data in HTML we first have to assemble all XML that
 : belong to a BdN work and then split it into smaller chunks (else each webpage
 : is too big to display it in a userfriendly way). The splitting is done by
 : considering two criteria:
 :
 : 1. which is the hierarchically highest tei:div of a work?
 : 2. does this tei:div have a tei:supplied[@type = "toc-title"] as descendant?
 :
 : ad 1: The hierarchical order of the tei:div is determined by its type: A part
 : may contain chapters and sections and a chapter may contain sections (part >
 : chapter > section). However, each work of the BdN has a slightly different
 : structure so we cannot simply assume that there is a tei:div[@type = "part"]
 : (some works may only have sections!).
 :
 : ad 2: For generating the navigation and generating HTML pages that make sense
 : in the context of a work, we also have to keep in mind the semantic structure
 : of a work that is indicated by tei:supplied[@type = "toc-title"]: Each unit
 : that has a tei:supplied[@type = "toc-title"] is a semantically coherent unit
 : that should be displayed together, thus should also be in an XML file together.
 :
 : Despite saving the whole and split XML files as well as their respective
 : intermediate format versions, we solve the whole serialization in memory in
 : order to avoid whitespace problems that we encounter when processing already
 : saved documents.
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 : @version 1.0
 :
 :)

xquery version "3.1";

module namespace xml-utils="http://bdn-edition.de/portal/xml-utils";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace xhtml="http://www.w3.org/1999/xhtml";
declare namespace html="http://www.w3.org/1999/html";
declare namespace xi="http://www.w3.org/2001/XInclude";
declare namespace textgrid="http://textgrid.info/namespaces/metadata/core/2010";
declare namespace bdn="http://www.bdn-edition.de/ns/1.0";

import module namespace config="https://sade.textgrid.de/ns/config" at "../config.xqm";
import module namespace functx="http://www.functx.com";
import module namespace transformKMG="http://mollenhauer-edition.de/ns/transform" at "../kmg/transform.xqm";

declare option exist:serialize "method=xml media-type=text/xml expand-xincludes=no";

(:~
 : Returns all xhtml:span[@class = "rdg-digital"] that appear in a node set as a
 : result of digital:transform.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $content the result of digital:transform applied to a file
 : @return all xhtml:span[@class = "rdg-digital"]
 :)
declare function xml-utils:find-rdgs-and-quotes($content as node()*) as node()*
{
    ($content//*[matches(@class, "rdg-digital") and @wits],
    $content//*[matches(@class, "quoted-text")])
};


(:~
 : Returns all non-xhtml:span[@class = ("rdg-digital", "quoted-text")] that
 : appear in a node set as a result of digital:transform.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $content the result of digital:transform applied to a file
 : @return all xhtml:span[@class = "rdg-digital"]
 :)
declare function xml-utils:find-non-rdgs-and-non-quotes($content as node()*) as node()*
{
    for $node in $content return
    typeswitch ( $node )
      case text() return
          $node
      case comment() return
          $node
      default return
          if ($node[matches(@class, "rdg-digital")
          or matches(@class, "rdg-digital-path")
          or matches(@class, "quoted-text")
          or matches(@class, "quote-path")]) then
              ()
          else
              element {$node/name()}
              {
                  $node/@*,
                  xml-utils:find-non-rdgs-and-non-quotes($node/node())
              }
};
