import React from 'react'
import {locale, otherLocale, tease, t, buildUrl} from './kmg'

export class ArchSearchResult extends React.Component {
  constructor(props) {
    super(props)

    this.clickMe = this.clickMe.bind(this)
  }

  isType(type) {
    const index = this.props.data._index
    if (type == 'register') {return !!index.match(/_(people|works|places)$/)}
    if (type == 'page') {return !!index.match(/_wiki_pages$/)}
    if (type == 'edition') {return !!index.match(/_editions$/)}
    if (type == 'commentary') {return !!index.match(/commentaries$/)}


    if (type == 'person') {return !!index.match(/_people$/)}
    if (type == 'work') {return !!index.match(/_works$/)}
    if (type == 'place') {return !!index.match(/_places$/)}

    return false
  }

  clickMe(event) {
    if (this.isType('register')) {
      event.preventDefault()
      this.openResult()
    }
  }

  editionFor(translation) {
    return [translation] || translation
  }

  resultUrl(version, kwic) {

    const source = this.props.data._source
    // build one string from highlightPatterns-array
    const highlight = this.highlightPatterns().join('|')
    // find context before/after
    const regex_before = new RegExp('(.*?)<span class="hl">(.*)<\/span>', 'gm')
    const before = regex_before.exec(kwic)[1]
    const regex_after = new RegExp('<span class="hl">(.*)<\/span>(.*)', 'gm')
    const after = regex_after.exec(kwic)[2]
    if (this.isType('edition') ) {
      return buildUrl('kmg.html', {
        params: {
          file: this.editionFor(source.edition),
          edition: version,
          lang: source.locale || 'de'
        },
        hashPath: '/',
        hashParams: {
          highlight: highlight,
          before: before,
          after: after
        }
      })
    }

    if (this.isType('commentary')) {
      return buildUrl('kmg.html', {
        params: {
          file: this.editionFor(source.edition),
          edition: "commentary",
          lang: source.locale || 'de'
        },
        hashPath: '/',
        hashParams: {
          highlight: highlight,
          before: before,
          after: after
        }
      })
    }

    if (this.isType('page')) {
      return buildUrl('content.html', {
        params: {
          id: source.id,
          lang: source.locale
        },
        hashPath: '/',
        hashParams: {
          highlight: highlight,
          before: before,
          after: after
        }
      })
    }

    return '#'
  }

  resultUrlOther() {
    const source = this.props.data._source

    if (this.isType('edition')) {

      let params = {
        //page: source.page_number,
        lang: source.locale || 'de'
      }

      if (params.lang == 'de') {
        Object.assign(params, {
          edition: source.edition,
          translation: this.translationFor(source.edition)
        })
      } else {
        Object.assign(params, {
          edition: this.editionFor(source.edition),
          translation: source.edition
        })
      }

      return buildUrl('view.html', {
        params: params,
        hashPath: '/',
        hashParams: {
          highlight: this.highlightPatterns().join('|')
        }
      })
    }

    if (this.isType('page')) {
      return buildUrl('content.html', {
        params: {
          id: source.id,
          lang: source.locale
        }
      })
    }

    return '#'
  }


  openResult() {
    if (this.isType('person')) {openPersName(this.props.data._source.id)}
    if (this.isType('work')) {openArtWork(this.props.data._source.id)}
    if (this.isType('place')) {openPlaceName(this.props.data._source.id)}
  }

  highlights_edition(version) {
    const list = this.props.data.highlight["search_data_" + version] || []
    if (list.length == 0) {return ''}

    const inner = list.map((e, i) => {
      return (
        <li
          key={i}
          className="pt-2 arch-text"
        >
          <a href={this.resultUrl(version, e)} dangerouslySetInnerHTML={{__html: e}}></a>
        </li>
      )
    })

    return <ul className="arch-highlights">{inner}</ul>
  }

  highlights_commentaries() {
    const list = this.props.data.highlight.search_data || []
    if (list.length == 0) {return ''}

    const inner = list.map((e, i) => {
      return (
        <li
          key={i}
          className="pt-2 arch-text"
        >
          <a href={this.resultUrl("commentary", e)} dangerouslySetInnerHTML={{__html: e}}></a>
        </li>
      )
    })

    return <ul className="arch-highlights">{inner}</ul>
  }

  highlights_pages() {
    const list = this.props.data.highlight.search_data || []
    if (list.length == 0) {return ''}

    const inner = list.map((e, i) => {
      return (
        <li
          key={i}
          className="pt-2 arch-text"
        >
          <a href={this.resultUrl("", e)} dangerouslySetInnerHTML={{__html: e}}></a>
        </li>
      )
    })

    return <ul className="arch-highlights">{inner}</ul>
  }

  highlights() {
    const list = this.props.data.highlight.search_data || []
    if (list.length == 0) {return ''}

    const inner = list.map((e, i) => {
      return (
        <li
          key={i}
          className="pt-2 arch-text"
        >
          <span dangerouslySetInnerHTML={{__html: e}}></span>
        </li>
      )
    })

    return <ul className="arch-highlights">{inner}</ul>
  }

  highlightPatterns() {
    let results = []
    const all = this.props.data.highlight
    for (const [k, v] of Object.entries(all)) {
      for (const hl of v) {
        // a serious hash to somewhat work around
        // https://github.com/elastic/elasticsearch/issues/29561, we use
        // combine adjacent matches so that the singluar parts are not carried
        // over as matches to the synoptic view
        const hl_combined = hl.matchAll(
          /<span class="hl">[^<]+<\/span>([\s\-\,\.]+<span class="hl">[^<]+<\/span>)*/g
        )
        for (const snippet of hl_combined) {
          const clean = snippet[0].replaceAll(/<[^>]+>/g, '')

          if (!results.includes(clean)) {
            results.push(clean)
          }
        }
      }
    }
    return results
  }

  name() {
    if (this.isType('register')) {
      const d = this.props.data

      const hl =
        d.highlight[`name.${locale()}`] ||
        d._source.name[locale()] ||
        d.highlight[`name.${otherLocale()}`] ||
        d._source.name[otherLocale()] ||
        []

      let result = hl

      // let result = hl[0] ||
      //   d._source.name[locale()] ||
      //   d._source.name[otherLocale()]

      // its an array if there are e.g. historical names (or empty tags for
      // historical names)
      if (Array.isArray(result)) {
        result = result.filter(x => !!x).join(', ')
      }

      return result
    }

    if (this.isType('edition')) {
      let base = (
        (this.props.data.highlight.name || [])[0] ||
        this.props.data._source.name ||
        `${this.props.data._source.id}`
      )
      return `${base}`
    }

    if (this.isType('commentary')) {
      let base = (
        (this.props.data.highlight.name || [])[0] ||
        this.props.data._source.name ||
        `${this.props.data._source.id}`
      )
      return `${base}`
    }

    if (this.isType('page')) {
      return this.props.data._source.name
    }
  }

  dataId() {
    return this.props.data._source.id
  }

  render() {

    let name = this.name() || t('button_unnamed')

    if (this.isType('edition')) {
      return(
        <div
          className="arch-search-result d-block mb-3 p-3"
          target="_blank"
          rel="noopener"
        >
          <h3
            className="arch-title"
            data-id={this.dataId()}
            dangerouslySetInnerHTML={{__html: name}}
          />
          <h5><em>{this.highlights_edition("a").length == 0 ? "": "Fassung a"}</em></h5>
          {this.highlights_edition("a")}
          <h5><em>{this.highlights_edition("b").length == 0 ? "": "Fassung b"}</em></h5>
          {this.highlights_edition("b")}
          <h5><em>{this.highlights_edition("c").length == 0 ? "": "Fassung c"}</em></h5>
          {this.highlights_edition("c")}
          <h5><em>{this.highlights_edition("d").length == 0 ? "": "Fassung d"}</em></h5>
          {this.highlights_edition("d")}
          <h5><em>{this.highlights_edition("e").length == 0 ? "": "Fassung e"}</em></h5>
          {this.highlights_edition("e")}
          <h5><em>{this.highlights_edition("f").length == 0 ? "": "Fassung f"}</em></h5>
          {this.highlights_edition("f")}
        </div>
      )
    }

    if (this.isType('commentary')) {
      return(
        <div
        className="arch-search-result d-block mb-3 p-3"
        target="_blank"
        rel="noopener"
        >
          <h3
            className="arch-title"
            data-id={this.dataId()}
            dangerouslySetInnerHTML={{__html: name}}
          />
          {this.highlights_commentaries()}
        </div>
      )
      }

    if (this.isType('register')) {
      return(
        <span
        className="arch-search-result d-block mb-3 p-3 js-editorial-info--person"
        // href={this.resultUrlOther()}
        data-id={this.dataId()}
        target="_blank"
        rel="noopener"
        // onClick={this.clickMe}
        >
        <em
            className="arch-title"
            dangerouslySetInnerHTML={{__html: name}}
          />
        </span>
      )
    }

    if (this.isType('page')) {
      return(
        <div
          className="arch-search-result d-block mb-3 p-3"
          target="_blank"
          rel="noopener"
          data-id={this.dataId()}
          //onClick={this.clickMe}
        >
          <h3
            className="arch-title"
            dangerouslySetInnerHTML={{__html: name}}
          />
          {this.highlights_pages()}
        </div>
      )
    }


  }
}
