/**
 * This function vertically positions a span containing a reading, either at the
 * same height as the corresponding lemma, or at a different height when it would
 * overlap other readings.
 *
 * @function
 * @param {object} rdg A span containing a reading.
 */
const positionRdg = function positionRdg(rdg) {
  let openRdgs = document.getElementsByClassName('rdg-digital--show');
  let lemmaId = rdg.getAttribute('target-lem').replace('#', '');
  let lemma = document.getElementById(lemmaId);
  let rdgPos = lemma.classList.contains('margin') ?
    lemma.parentElement.offsetTop + 238
    : lemma.offsetTop;
  let rdgBottom = rdgPos + rdg.offsetHeight;

  for (openedRdg of openRdgs) {
    if (openedRdg != rdg) {
      let openedRdgBottom = openedRdg.offsetTop + openedRdg.offsetHeight;
      rdgPos >= openedRdg.offsetTop && rdgPos < openedRdgBottom + 5 || rdgPos < openedRdg.offsetTop - 5 && rdgBottom > openedRdgBottom + 5
        ? rdgPos = openedRdgBottom + 5
        : rdgPos;
    }
  };
  rdgBottom = rdgPos + rdg.offsetHeight;
  for (openedRdg of openRdgs) {
    if (openedRdg != rdg) {
      let openedRdgBottom = openedRdg.offsetTop + openedRdg.offsetHeight;
      rdgBottom > openedRdg.offsetTop - 5 && rdgBottom <= openedRdgBottom || rdgPos < openedRdg.offsetTop - 5 && rdgBottom > openedRdgBottom + 5
        ? rdgPos = openedRdgBottom + 5
        : rdgPos;
    }
  };
  rdg.style.top = `${rdgPos}px`;
  if (rdg.classList.contains('varying-structure')) {
    rdg.style.maxHeight = `${lemma.offsetHeight}px`;
  };
};

/**
 * This function opens a reading, positions it, highlights the corresponding lemma.
 * This function is called when one of the
 * variant buttons is clicked.
 *
 * @function
 * @param {object} rdg A span containing a reading.
 */
const openRdg = function openRdg(rdg) {
  let lemmaId = rdg.getAttribute('target-lem').replace('#', '');
  let lemma = document.getElementById(lemmaId);

  lemma.classList.add('lemma--open');
  positionRdg(rdg);
  rdg.classList.add('rdg-digital--show');
};

/**
 * This function closes a reading when the reading, its corresponding lemma or
 * the variant button is clicked.
 *
 * @function
 * @param {object} rdg A span containing a reading.
 */
const closeRdg = function closeRdg(rdg) {
  event.stopPropagation();
  let lemmaId = rdg.getAttribute('target-lem').replace('#', '');
  let lemma = document.getElementById(lemmaId);

  /** do not remove highlighting from corresponding lemma if there is another
     * open reading corresponding to that lemma.
     */
  document.querySelectorAll(`.rdg-digital--show[target-lem='#${lemmaId}']`).length > 1 ? '' : lemma.classList.remove('lemma--open');
  rdg.classList.remove('rdg-digital--show');
  rdg.style.top = '';
};

/**
 * This function handles the highlighting of a lemma when clicked and calls the
 * display of any relevant readings.
 *
 * @function
 * @param {object} lem A span containing a lemma.
 */
const openLemma = function openLemma(lem) {
  event.stopPropagation();
  let id = lem.id;
  let relevantRdgs = document.querySelectorAll(`.rdg-digital[target-lem='#${id}']`);

  lem.classList.add('lemma--open');
  for (rdg of relevantRdgs) {
    positionRdg(rdg);
    rdg.classList.add('rdg-digital--show');
  };
};

/**
 * This function closes a lemma and all its corresponding readings.
 *
 * @function
 * @param {object} lem A span containing a lemma.
 */
const closeLemma = function closeLemma(lem) {
  event.stopPropagation();
  let id = lem.id;
  let relevantRdgs = document.querySelectorAll(`.rdg-digital[target-lem='#${id}']`);

  lem.classList.remove('lemma--open');
  for (rdg of relevantRdgs) {
    closeRdg(rdg);
  };
};

/**
 * This is a wrapper function, checking whether a clicked lemma is open or closed.
 *
 * @function
 */
const toggleLemma = function toggleLemma() {
  this.classList.contains('lemma--open') ? closeLemma(this) : openLemma(this);
};


/**
 * This function highlights a lemma's correspoding reading or
 * readings on mouseover.
 *
 * @function
 */
const mouseOverLem = function mouseOverLem() {
  let id = this.id;
  let rdg = document.querySelector(`[target-lem='#${id}']`);

  rdg.classList.toggle('highlight');
};

/**
 * This function highlights a reading on mouseover, as well as its corresponding
 * lemma.
 *
 * @function
 */
const mouseOverRdg = function mouseOverRdg() {
  let lemId = this.getAttribute('target-lem').replace('#', '');
  let lem = document.getElementById(lemId);

  this.classList.toggle('highlight');
  lem.classList.toggle('highlight');
};

/**
 * This function sets the position
 * of any opened readings when the screen is
 * resized.
 *
 * @function
 */
const onScreenResize = function onScreenResize() {
  let openRdgs = document.querySelectorAll('.rdg-digital--show');

  for (rdg of openRdgs) {
    let lemmaId = rdg.getAttribute('target-lem').replace('#', '');
    let lemma = document.getElementById(lemmaId);
    positionRdg(rdg);
  };
};

/**
 * Get all lemmata and readings.
 */
const lems = document.getElementsByClassName('lemma');
const rdgs = document.getElementsByClassName('rdg-digital');

window.addEventListener('resize', onScreenResize);

for (i = 0; i < lems.length; i++) {
  lems[i].addEventListener('click', toggleLemma, false);
  lems[i].addEventListener('mouseenter', mouseOverLem, false);
  lems[i].addEventListener('mouseleave', mouseOverLem, false);
};

for (i = 0; i < rdgs.length; i++) {
  rdgs[i].addEventListener('click', function(){closeRdg(this);}, false);
  rdgs[i].addEventListener('mouseenter', mouseOverRdg, false);
  rdgs[i].addEventListener('mouseleave', mouseOverRdg, false);
};
