import 'bootstrap/js/dist/dropdown';
import 'bootstrap/js/dist/modal';
import 'bootstrap/js/dist/popover';
import 'bootstrap/js/dist/button';
import '@fortawesome/fontawesome-free/js/all';

import './kmg-interactive';
import {
  setAuth, setSid, addTGObject, getMimeClass, view, links, allowDrop, drop, log, publish
} from './publish-gui';
import './edition';
import './edition-nav';
import './register';
import './search';

/* PUBLISH */
/* add functions to the global scope so that they can be called by the TGLab plugin */
window.setAuth = setAuth;
window.setSid = setSid;
window.addTGObject = addTGObject;
window.getMimeClass = getMimeClass;
window.view = view;
window.links = links;
window.allowDrop = allowDrop;
window.drop = drop;
window.log = log;
window.publish = publish;

const dropdown = document.querySelectorAll('button.dropdown');
dropdown.forEach((e) => e.addEventListener('click', function() {
  let expanded = this.getAttribute('aria-expanded') === 'true' || false;
  this.setAttribute('aria-expanded', !expanded);
  this.classList.toggle('burger-open');
  let nav = this.parentNode;
  let menu = nav.querySelector('ul');
  nav.classList.toggle('bg-primary');
  menu.classList.toggle('hidden');
})
);

const scrolltop = document.querySelector('.scrolltop')
window.addEventListener('scroll', () => {
  if (window.scrollY > 170) {
    scrolltop.classList.remove('hidden');
  } else {
    scrolltop.classList.add('hidden');
  }
});

/**
 * Functionality to switch between light and dark theme. Stores system preference
 * and chosen mode in local storage.
 */
const darkmodeBtn = document.querySelector('.darktheme-button');
const prefersDarkTheme = window.matchMedia('(prefers-color-scheme: dark)');

const currentTheme = localStorage.getItem('theme');
if (currentTheme == 'dark') {
  document.body.classList.toggle('dark-theme');
} else if (currentTheme == 'light') {
  document.body.classList.toggle('light-theme');
}

darkmodeBtn.addEventListener('click', () => {
  let theme;
  document.body.classList.toggle('dark-theme');
  document.body.classList.toggle('light-theme');
  if (prefersDarkTheme.matches) {
    theme = document.body.classList.contains('light-theme')
      ? 'light'
      : 'dark';
  } else {
    theme = document.body.classList.contains('dark-theme')
      ? 'dark'
      : 'light';
  }
  localStorage.setItem('theme', theme);
});

const accordions = document.querySelectorAll('.accordion-button');

Array.prototype.forEach.call(accordions, (a) => {
  const content = a.parentNode.nextElementSibling;

  a.onclick = () => {
    const expanded = a.getAttribute('aria-expanded') === 'true';
    a.setAttribute('aria-expanded', !expanded);
    expanded === 'true' ? content.classList.remove('hidden') : content.classList.add('hidden');
  }
});

(function() {
  // Check for <template> support
  if ('content' in document.createElement('template')) {
    const tmpl = document.createElement('template')

    // Create the web component's template
    // featuring a <slot> for the Light DOM content
    tmpl.innerHTML = `
      <h2>
        <button aria-expanded="false" class="accordion-button">
          <svg aria-hidden="true" focusable="false" width="40" height="40" viewBox="0 0 80 80" fill="none" xmlns="http://www.w3.org/2000/svg">
              <path d="M30.9862 47L49.0867 47C50.9133 47 51.828 44.7916 50.5364 43.5L41.8926 34.8562C40.8675 33.831 39.2054 33.831 38.1803 34.8562L29.5364 43.5C28.2449 44.7916 29.1596 47 30.9862 47Z" fill="var(--color-primary)" stroke="var(--color-primary)" stroke-width="4" stroke-linecap="round" stroke-linejoin="round" />
          </svg>
        </button>
      </h2>
      <div class="content" hidden>
        <slot></slot>
      </div>
      <style>
        h2 {
          background-color: var(--color-background-contrast);
          cursor: pointer;
          height: 50px;
          margin: 5px 0 0;
          font: normal normal 18px/20px "Open Sans";
        }

        h2 button {
          all: inherit;
          align-items: center;
          box-sizing: border-box;
          display: flex;
          justify-content: space-between;
          width: 100%;
          padding: 0 13px 0 25px;
        }

        h2 button:focus:focus-visible svg {
          outline: 4px solid var(--color-contrast);
        }

        button svg {
          height: 2em;
          margin-left: 0.5em;
        }

        [aria-expanded="true"] {
          background: var(--color-primary);
          color: var(--color-background);
        }

        [aria-expanded="false"] svg {
          transform: rotate(180deg);
        }

        [aria-expanded="true"] svg {
          --color-primary: var(--color-background);
        }
      </style>
    `
    // Check for latest Shadow DOM syntax support
    if (document.head.attachShadow) {
      class ToggleSection extends HTMLElement {
        constructor() {
          super()

          // Make the host element a region
          this.setAttribute('role', 'region')

          // Create a `shadowRoot` and populate from template
          this.attachShadow({ mode: 'open' })
          this.shadowRoot.appendChild(tmpl.content.cloneNode(true))

          // Assign the toggle button
          this.btn = this.shadowRoot.querySelector('h2 button')

          // Get the first element in Light DOM
          const oldHeading = this.querySelector(':first-child')
          // and cast its heading level (which should, but may not, exist)
          let level = parseInt(oldHeading.tagName.substr(1))
          // Then take its `id` (may be null)
          let id = oldHeading.id

          // Get the Shadow DOM <h2>
          this.heading = this.shadowRoot.querySelector('h2')

          // If `id` exists, apply it
          if (id) {
            this.heading.id = id
          }

          // If there is no level, there is no heading.
          // Add a warning.
          if (!level) {
            console.warn('The first element inside each <toggle-section> should be a heading of an appropriate level.')
          }

          // If the level is a real integer but not 2
          // set `aria-level` accordingly
          if (level && level !== 2) {
            this.heading.setAttribute('aria-level', level)
          }

          // Add the Light DOM heading label to the innerHTML of the toggle button
          // and remove the now unwanted Light DOM heading
          this.btn.innerHTML = oldHeading.textContent + this.btn.innerHTML
          oldHeading.parentNode.removeChild(oldHeading)

          // The main state switching function
          this.switchState = () => {
            let expanded = this.getAttribute('open') === 'true' || false

            // Toggle `aria-expanded`
            this.btn.setAttribute('aria-expanded', expanded)
            // Toggle the `.content` element's visibility
            this.shadowRoot.querySelector('.content').hidden = !expanded
          }

          this.btn.onclick = () => {
            // Change the component's `open` attribute value on click
            let open = this.getAttribute('open') === 'true' || false
            this.setAttribute('open', open ? 'false' : 'true')

            // Update the hash if the collapsible section's
            // heading has an `id` and we are opening, not closing
            if (this.heading.id && !open) {
              history.pushState(null, null, '#' + this.heading.id)
            }
          }
        }

        connectedCallback() {
          if (window.location.hash.substr(1) === this.heading.id) {
            this.setAttribute('open', 'true')
            this.btn.focus(focusVisible)
          }
        }

        // Identify just the `open` attribute as an observed attribute
        static get observedAttributes() {
          return ['open']
        }

        // When `open` changes value, execute switchState()
        attributeChangedCallback(name) {
          if (name === 'open') {
            this.switchState()
          }
        }
      }

      // Add our new custom element to the window for use
      window.customElements.define('toggle-section', ToggleSection)

    }
  }
})()
