xquery version "3.1";

(: ~
 : This module contains unit tests for kmg-rest.xqm.
 :
 : @author Simon Sendler
 : @since 1.3.0
 :)

module namespace tstapi="http://mollenhauer-edition.de/ns/tstapi";

import module namespace test="http://exist-db.org/xquery/xqsuite" at "resource:org/exist/xquery/lib/xqsuite/xqsuite.xql";
import module namespace kmg-rest="https://mollenhauer-edition.de/ns/kmg-rest" at "../modules/kmg/kmg-rest.xqm";

declare
    %test:args("2021-04-23T15:37:25.176+02:00") %test:assertEquals("23.4.2021, 15:37")
    %test:args("2021-10-02T15:07:01.17+02:00") %test:assertEquals("2.10.2021, 15:7")
function tstapi:date-to-string($date as xs:dateTime)
as xs:string
{
    kmg-rest:date-to-string($date)
};

declare
    %test:assertExists
    %test:asserEquals(
        "23.4.2021, 15:37",
        "2021-04-23T15:37:25.176+02:00",
        "could not retrieve date of last transformation for object with URI 12345",
        "Soziale Arbeit heute. Gedanken über ihre sozialen und ideologischen Voraussetzungen",
        "textgrid:3qqv4.3")
function tstapi:format-metadata()
as xs:string+
{
    let $map := kmg-rest:format-metadata(local:sample-TG-metadata(), "12345")
    let $created := map:get($map, "created")
    let $transformed := map:get($map, "transformed")
    let $title := map:get($map, "title")
    let $tguri := map:get($map, "tguri")
    let $result := 
        (map:get($created, "string"),
        map:get($created, "raw"),
        map:get($transformed, "error"),
        $title,
        $tguri)
    return
        $result
};

declare
    %test:assertExists
    %test:assertEquals(
        "1959", 
        "main", 
        "version", 
        "Sammelbandbeitrag 1998", 
        "Distributed under the Creative Commons Attribution-ShareAlike 4.0 Generic (International) License.", 
        "https://creativecommons.org/licenses/by-sa/4.0/",
        "Soziale Arbeit heute"
    )
function tstapi:format-teiHeader()
as xs:string+
{
    let $map := kmg-rest:format-data(local:sample-teiHeader())
    let $witnesses := map:get($map, "witnesses")
    let $licence := map:get($map, "licence")
    let $title := map:get($map, "title")
    let $result := 
        (map:get($witnesses, "a") => map:get("year"),
        map:get($witnesses, "a") => map:get("type"),
        map:get($witnesses, "b") => map:get("type"),
        map:get($witnesses, "b") => map:get("description"),
        map:get($licence, "name"),
        map:get($licence, "url"),
        map:get($title, "main"))
    return
        $result
};

declare
    %test:assertExists
    %test:assertEquals(
        "Wolgast",
        "Philipp Otto Runge (Maler)",
        "Hamburger Kunsthalle",
        "https://www.hamburger-kunsthalle.de/",
        "Gemälde",
        "196 x 131 cm"
    )
function tstapi:format-lido()
as xs:string+
{
    let $map := kmg-rest:format-lido(local:sample-lido())
    let $repository := map:get($map, "repository")
    let $result :=
        (map:get($map, "origin"),
        map:get($map, "artist") => map:get("name"),
        map:get($repository, "name"),
        map:get($repository, "href"),
        map:get($map, "worktype") => map:get("term"),
        map:get($map, "measurements"))
    return
        $result
};

declare function local:sample-TG-metadata()
as node()
{
    <MetadataContainerType xmlns="http://textgrid.info/namespaces/metadata/core/2010">
        <object>
            <generic>
                <provided>
                    <title>1959-Soziale Arbeit heute. Gedanken über ihre sozialen und ideologischen Voraussetzungen</title>
                    <format>text/xml</format>
                    <notes/>
                </provided>
                <generated>
                    <created>2021-04-23T15:37:25.176+02:00</created>
                    <lastModified>2021-04-23T15:37:25.176+02:00</lastModified>
                    <textgridUri extRef="">textgrid:3qqv4.3</textgridUri>
                    <revision>3</revision>
                    <extent>169411</extent>
                    <fixity>
                        <messageDigestAlgorithm>md5</messageDigestAlgorithm>
                        <messageDigest>ae2e9687e52c46e263b7f60633a0fadb</messageDigest>
                        <messageDigestOriginator>crud-base 10.2.5.2-TG</messageDigestOriginator>
                    </fixity>
                    <dataContributor>MaxZeterberg@dariah.eu</dataContributor>
                    <project id="TGPR-4d929bb8-db00-b972-da3a-58b452527262">KMG</project>
                    <permissions>read write delete delegate publish</permissions>
                </generated>
            </generic>
        </object>
    </MetadataContainerType>
};

declare function local:sample-teiHeader()
as node()
{
    <teiHeader xml:lang="de-1996" xmlns="http://www.tei-c.org/ns/1.0">
        <fileDesc>
            <titleStmt>
                <title type="main">Soziale Arbeit heute</title>
                <title type="sub">
                    Gedanken über ihre sozialen und ideologischen Voraussetzungen
                </title>
            </titleStmt>
            <publicationStmt>
                <availability>
                    <licence target="https://creativecommons.org/licenses/by-sa/4.0/">
                        Distributed under the Creative Commons Attribution-ShareAlike 4.0 Generic (International) License.
                    </licence>
                </availability>
                <idno type="URI">textgrid:3qqv4</idno>
            </publicationStmt>
            <sourceDesc>
                <listWit>
                    <witness xml:lang="de-1901" xml:id="a" rend="qm_low spaced">
                        <desc type="Leittext Version">Zeitschriftenartikel 1959</desc>
                        <listWit>
                            <witness>
                                <desc type="Leittext article_journal">
                                    Beitrag in der Zeitschrift Gilde Rundbrief 1959
                                </desc>
                                <bibl corresp="zotero:4H8LHM2S"/>
                            </witness>
                        </listWit>
                    </witness>
                    <witness xml:lang="de-1901" xml:id="b" rend="qm_low italics">
                        <desc type="Version">Sammelbandbeitrag 1998</desc>
                        <listWit>
                            <witness>
                                <desc type="article_volume">
                                    Beitrag im Sammelband Sozialarbeit – Sozialpädagogik – soziale Arbeit 1998
                                </desc>
                                <bibl corresp="zotero:NJPCJMWI"/>
                            </witness>
                        </listWit>
                    </witness>
                </listWit>
            </sourceDesc>
        </fileDesc>
    </teiHeader>
};

declare function local:sample-lido()
as node()
{
    <lido:lido xmlns:p="https://schema.easydb.de/EASYDB/1.0/objects/" xmlns:lido="http://www.lido-schema.org" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.lido-schema.org http://www.lido-schema.org/schema/v1.0/lido-v1.0.xsd">
        <lido:lidoRecID lido:type="http://terminology.lido-schema.org/identifier_type/local_identifier">textgrid:3r2f8</lido:lidoRecID>
        <lido:descriptiveMetadata xml:lang="de">
            <lido:objectClassificationWrap>
                <lido:objectWorkTypeWrap>
                    <lido:objectWorkType>
                        <lido:conceptID lido:type="http://terminology.lido-schema.org/identifier_type/uri">https://d-nb.info/gnd/4122164-3</lido:conceptID>
                        <lido:term>Gemälde</lido:term>
                    </lido:objectWorkType>
                </lido:objectWorkTypeWrap>
            </lido:objectClassificationWrap>
            <lido:objectIdentificationWrap>
                <lido:titleWrap>
                    <lido:titleSet>
                        <lido:appellationValue>Die Eltern des Künstlers</lido:appellationValue>
                    </lido:titleSet>
                </lido:titleWrap>
                <lido:repositoryWrap>
                    <lido:repositorySet lido:type="http://terminology.lido-schema.org/repositorySet_type/current_repository_or_location">
                        <lido:repositoryName>
                            <lido:legalBodyName>
                                <lido:appellationValue>Hamburger Kunsthalle</lido:appellationValue>
                            </lido:legalBodyName>
                            <lido:legalBodyWeblink>https://www.hamburger-kunsthalle.de/</lido:legalBodyWeblink>
                        </lido:repositoryName>
                        <lido:workID>HK-1001</lido:workID>
                        <lido:repositoryLocation>
                            <lido:namePlaceSet>
                                <lido:appellationValue>Hamburg</lido:appellationValue>
                            </lido:namePlaceSet>
                        </lido:repositoryLocation>
                    </lido:repositorySet>
                </lido:repositoryWrap>
                <lido:objectDescriptionWrap>
                    <lido:objectDescriptionSet>
                        <lido:descriptiveNoteValue>Das Gemälde Die Eltern des Künstlers von Philipp Otto Runge aus dem Jahr 1806 ist ein hochformatiges Ganzkörper-Porträt im Stil der deutschen Romantik. Aufgrund der Inschrift auf der Rückseite der Leinwand, geht die Zuordnung der Personen als Daniel Nicolaus Runge, Vater und Magdalena Dorothea Runge, Mutter des Künstlers Runge, sowie seinem ersten Sohn Otto Sigismund und seinem Neffen Friedrich hervor. Der Künstler wählt für die ältere Generation dunkle Farben, für die Kinder helle. Im Vordergrund des Bildes befinden sich verschiedene Pflanzen und Steine, der linke Hintergrund wird durch eine grüne Hauswand begrenzt. Einzig auf der rechten Seite, über einem Zaun, ist ein Blick in die Tiefe, eine Seenlandschaft mit Schiffen und Wäldern, sowie einem bewölkten Himmel erkennbar. Das Bild wirkt durch die ruhige Farbgebung und den weichen Pinselstrich harmonisch, Details sind z. B. im Faltenwurf des Mantels und im Gesicht der Mutter fein ausgearbeitet.</lido:descriptiveNoteValue>
                    </lido:objectDescriptionSet>
                </lido:objectDescriptionWrap>
                <lido:objectMeasurementsWrap>
                    <lido:objectMeasurementsSet>
                        <lido:displayObjectMeasurements>196 x 131 cm</lido:displayObjectMeasurements>
                    </lido:objectMeasurementsSet>
                </lido:objectMeasurementsWrap>
            </lido:objectIdentificationWrap>
            <lido:eventWrap>
                <lido:eventSet>
                    <lido:event>
                        <lido:eventType>
                            <lido:conceptID lido:type="http://terminology.lido-schema.org/identifier_type/uri">http://terminology.lido-schema.org/eventType/beginning_of_existence</lido:conceptID>
                            <lido:term xml:lang="en">Beginning of existence</lido:term>
                            <lido:term>Daseinsbeginn</lido:term>
                        </lido:eventType>
                        <lido:eventActor>
                            <lido:displayActorInRole>Philipp Otto Runge (Maler)</lido:displayActorInRole>
                            <lido:actorInRole>
                                <lido:actor>
                                    <lido:actorID lido:type="http://terminology.lido-schema.org/identifier_type/uri">https://d-nb.info/gnd/118604155</lido:actorID>
                                    <lido:actorID lido:type="http://terminology.lido-schema.org/identifier_type/local_identifier">POR</lido:actorID>
                                    <lido:nameActorSet>
                                        <lido:appellationValue>Philipp Otto Runge</lido:appellationValue>
                                    </lido:nameActorSet>
                                </lido:actor>
                            </lido:actorInRole>
                        </lido:eventActor>
                        <lido:eventDate>
                            <lido:displayDate>1806</lido:displayDate>
                        </lido:eventDate>
                        <lido:eventPlace>
                            <lido:displayPlace>Wolgast</lido:displayPlace>
                            <lido:place>
                                <lido:placeID lido:type="http://terminology.lido-schema.org/identifier_type/uri">https://www.geonames.org/2806501/wolgast.html</lido:placeID>
                                <lido:namePlaceSet>
                                    <lido:appellationValue>Wolgast</lido:appellationValue>
                                </lido:namePlaceSet>
                            </lido:place>
                        </lido:eventPlace>
                        <lido:eventMaterialsTech>
                            <lido:displayMaterialsTech>Öl auf Leinwand</lido:displayMaterialsTech>
                        </lido:eventMaterialsTech>
                    </lido:event>
                </lido:eventSet>
            </lido:eventWrap>
            <lido:objectRelationWrap>
                <lido:relatedWorksWrap>
                    <lido:relatedWorkSet>
                        <lido:relatedWork>
                            <lido:object>
                                <lido:objectID lido:type="http://terminology.lido-schema.org/identifier_type/uri">https://online-sammlung.hamburger-kunsthalle.de/de/objekt/HK-1001</lido:objectID>
                            </lido:object>
                        </lido:relatedWork>
                    </lido:relatedWorkSet>
                    <lido:relatedWorkSet>
                        <lido:relatedWork>
                            <lido:object>
                                <lido:objectWebResource>https://online-sammlung.hamburger-kunsthalle.de/de/objekt/HK-1001</lido:objectWebResource>
                            </lido:object>
                        </lido:relatedWork>
                        <lido:relatedWorkRelType>
                            <lido:term>Weblink</lido:term>
                        </lido:relatedWorkRelType>
                    </lido:relatedWorkSet>
                </lido:relatedWorksWrap>
            </lido:objectRelationWrap>
        </lido:descriptiveMetadata>
    </lido:lido>
};
